import aiohttp
import requests

from contextlib import asynccontextmanager, contextmanager
from typing import Dict, Any, AsyncGenerator, Generator, Optional


class AutoFaqHTTPClient:
    def __init__(self, base_url: str = "", timeout: int = 30):
        self.base_url = base_url
        self.timeout = timeout
        self._sync_session: Optional[requests.Session] = None
        self._async_session: Optional[aiohttp.ClientSession] = None
        self._default_headers: Dict[str, str] = {}
    
    @property
    def sync_session(self) -> requests.Session:
        if self._sync_session is None:
            self._sync_session = requests.Session()
            if self._default_headers:
                self._sync_session.headers.update(self._default_headers)
        return self._sync_session
    
    @property
    def async_session(self) -> aiohttp.ClientSession:
        if self._async_session is None:
            timeout = aiohttp.ClientTimeout(total=self.timeout)
            self._async_session = aiohttp.ClientSession(
                headers=self._default_headers.copy(), 
                timeout=timeout
            )
        return self._async_session
    
    def set_default_headers(self, headers: Dict[str, str]) -> None:
        self._default_headers.update(headers)
        if self._sync_session: self._sync_session.headers.update(headers)
            
    async def async_set_default_headers(self, headers: Dict[str, str]) -> None:
        self._default_headers.update(headers)
        
        if self._async_session:
            await self._async_session.close()
            self._async_session = None
    
    def update_default_headers(self, headers: Dict[str, str]) -> None:
        self.set_default_headers(headers)
    
    def clear_default_headers(self) -> None:
        self._default_headers.clear()
        if self._sync_session:
            self._sync_session.headers.clear()
            
    async def async_clear_default_headers(self) -> None:
        self._default_headers.clear()
        if self._async_session:
            if self._async_session:
                await self._async_session.close()
                self._async_session = None
    
    @contextmanager
    def temporary_sync_session(self, headers: Dict[str, str] = None) -> Generator[requests.Session, None, None]:
        session = requests.Session()
        if headers:
            session.headers.update(headers)
        elif self._default_headers:
            session.headers.update(self._default_headers)
            
        try:
            yield session
        finally:
            session.close()
    
    @asynccontextmanager
    async def temporary_async_session(self, headers: Dict[str, str] = None) -> AsyncGenerator[aiohttp.ClientSession, None]:
        timeout = aiohttp.ClientTimeout(total=self.timeout)
        session_headers = self._default_headers.copy()
        if headers:
            session_headers.update(headers)
            
        async with aiohttp.ClientSession(
            headers=session_headers, 
            timeout=timeout
        ) as session:
            yield session
    
    def _build_url(self, endpoint: str) -> str:
        return f"{self.base_url}/{endpoint.lstrip('/')}" if self.base_url else endpoint
    
    def _handle_response(self, response: requests.Response, return_json: bool = True) -> Any:
        self._check_status(response)
        if return_json: 
            try: return response.json()
            except: return response.content
        return response.content
    
    async def _handle_async_response(self, response: aiohttp.ClientResponse, return_json: bool = True) -> Any:
        await self._check_async_status(response)
        if return_json: 
            try: return await response.json()
            except: return await response.content.read()
        return await response.content.read()
    
    def _check_status(self, response: requests.Response) -> None:
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            try:
                error_text = response.text
                error_json = response.json()
                error_details = f"Response JSON: {error_json}"
            except:
                error_details = f"Response text: {error_text}"
            
            raise requests.exceptions.HTTPError(
                f"{str(e)}\n{error_details}", 
                response=response
            ) from e
    
    async def _check_async_status(self, response: aiohttp.ClientResponse) -> None:
        if response.status >= 400:
            try:
                error_text = await response.text()
                try:
                    error_json = await response.json()
                    error_details = f"Response JSON: {error_json}"
                except:
                    error_details = f"Response text: {error_text}"
            except Exception as e:
                error_details = f"Could not read response: {str(e)}"
            
            raise aiohttp.ClientResponseError(
                request_info=response.request_info,
                history=response.history,
                status=response.status,
                message=f"HTTP {response.status}: {response.reason}\n{error_details}",
                headers=response.headers
            )
    
    def close_sync_session(self) -> None:
        if self._sync_session:
            self._sync_session.close()
            self._sync_session = None
    
    async def close_async_session(self) -> None:
        if self._async_session:
            await self._async_session.close()
            self._async_session = None
    
    async def close(self) -> None:
        self.close_sync_session()
        await self.close_async_session()
    
    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close_sync_session()
    
    async def __aenter__(self):
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()
    
    def sync_get(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.get(url, **kwargs)
        return self._handle_response(response, return_json)
    
    def sync_post(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.post(url, **kwargs)
        return self._handle_response(response, return_json)
    
    def sync_put(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.put(url, **kwargs)
        return self._handle_response(response, return_json)
    
    def sync_patch(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.patch(url, **kwargs)
        return self._handle_response(response, return_json)
    
    def sync_delete(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.delete(url, **kwargs)
        return self._handle_response(response, return_json)
    
    def sync_head(self, endpoint: str, **kwargs) -> Dict[str, str]:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.head(url, **kwargs)
        self._check_status(response)
        return dict(response.headers)
    
    def sync_options(self, endpoint: str, **kwargs) -> Dict[str, Any]:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.options(url, **kwargs)
        self._check_status(response)
        return {
            'headers': dict(response.headers),
            'allowed_methods': response.headers.get('allow', '').split(', ')
        }
    
    async def async_get(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        async with self.async_session.get(url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)
    
    async def async_post(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        async with self.async_session.post(url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)
    
    async def async_put(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        async with self.async_session.put(url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)
    
    async def async_patch(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        async with self.async_session.patch(url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)
    
    async def async_delete(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        async with self.async_session.delete(url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)
    
    async def async_head(self, endpoint: str, **kwargs) -> Dict[str, str]:
        url = self._build_url(endpoint)
        async with self.async_session.head(url, **kwargs) as response:
            await self._check_async_status(response)
            return dict(response.headers)
    
    async def async_options(self, endpoint: str, **kwargs) -> Dict[str, Any]:
        url = self._build_url(endpoint)
        async with self.async_session.options(url, **kwargs) as response:
            await self._check_async_status(response)
            return {
                'headers': dict(response.headers),
                'allowed_methods': response.headers.get('allow', '').split(', ')
            }
    
    def sync_get_temp(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        with self.temporary_sync_session() as session:
            url = self._build_url(endpoint)
            response = session.get(url, **kwargs)
            return self._handle_response(response, return_json)
    
    def sync_post_temp(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        with self.temporary_sync_session() as session:
            url = self._build_url(endpoint)
            response = session.post(url, **kwargs)
            return self._handle_response(response, return_json)
    
    async def async_get_temp(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        async with self.temporary_async_session() as session:
            url = self._build_url(endpoint)
            async with session.get(url, **kwargs) as response:
                return await self._handle_async_response(response, return_json)
    
    async def async_post_temp(self, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        async with self.temporary_async_session() as session:
            url = self._build_url(endpoint)
            async with session.post(url, **kwargs) as response:
                return await self._handle_async_response(response, return_json)
    
    def sync_request(self, method: str, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.request(method.upper(), url, **kwargs)
        return self._handle_response(response, return_json)
    
    async def async_request(self, method: str, endpoint: str, return_json: bool = True, **kwargs) -> Any:
        url = self._build_url(endpoint)
        async with self.async_session.request(method.upper(), url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)


from pydantic import BaseModel, Field
from typing import List


class QueryModel(BaseModel):
    service_id: str = Field(..., min_length=1)
    service_token: str = Field(..., min_length=1)
    query: str = Field(..., min_length=1)
    top_k: int = Field(default=3, ge=1)
    session_id: str = Field(default="")
    intents: List[str] = Field(default=[])
    context_document_id: int = Field(default=0, ge=0)
    lower_bound: float = Field(default=0.0, le=1.0)
    
class BatchQueryModel(BaseModel):
    payload: List[QueryModel] = Field(..., min_items=1, max_items=100)



import os
import mimetypes
import aiofiles
import warnings

from base64 import b64encode
from aiohttp import FormData, StreamReader
from requests_toolbelt.multipart.encoder import MultipartEncoder
from pydantic import ValidationError
from .models.kb_external_models import (
    QuestionModel, 
    CloseСonversationModel, 
    GetConversationsModel, 
    PostDelayedDeliveryModel,
    VarModel, 
    ConversationsCountReportModel, 
    OperatorsReportModel
)
from .http_client import AutoFaqHTTPClient


class AutoFaqExternal(AutoFaqHTTPClient):
    
    def __init__(self, base_url:str, userlogin:str, userpassword:str, service_id:str, timeout: int = 30):
        """
        Инициализация клиента для работы с AutoFAQ External API.
        
        Args:
            base_url (str): Базовый URL API AutoFAQ
            userlogin (str): Логин пользователя для аутентификации
            userpassword (str): Пароль пользователя для аутентификации
            service_id (str): ID сервиса AutoFAQ
            timeout (int, optional): Таймаут запросов в секундах. По умолчанию 30.
        """
        super().__init__(base_url, timeout)
        self.userlogin = userlogin
        self.userpassword = userpassword
        self.service_id = service_id
        self._auth_in_progress = False
        
    def sync_authorization(self):
        """
        Синхронная аутентификация в API AutoFAQ.
        
        Получает Bearer token используя Basic Auth и сохраняет его для последующих запросов.
        
        Raises:
            Exception: Если аутентификация не удалась
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Аутентификация/Authenticate
        """
        basic_auth_data = f'{self.userlogin}:{self.userpassword}'.encode('utf-8')
        headers = { "Authorization": f"Basic {b64encode(basic_auth_data).decode()}"}
        url = self._build_url('/api/ext/v2/login')
        resp = self.sync_session.request('GET', url, headers=headers)
        is_response_json_serialize = False
        try:
            resp.json()
            is_response_json_serialize = True
        except: pass
        if is_response_json_serialize: raise Exception(f'Athorization error in "{url}"')
        auth_token = resp.text
        if auth_token:
            self.update_default_headers({"authorization": f"Bearer {auth_token}"}) 
            
    async def async_authorization(self):
        """
        Асинхронная аутентификация в API AutoFAQ.
        
        Получает Bearer token используя Basic Auth и сохраняет его для последующих запросов.
        
        Raises:
            Exception: Если аутентификация не удалась
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Аутентификация/Authenticate
        """
        basic_auth_data = f'{self.userlogin}:{self.userpassword}'.encode('utf-8')
        headers = {"Authorization": f"Basic {b64encode(basic_auth_data).decode()}"}
        try: 
            url = self._build_url('/api/ext/v2/login')
            async with self.async_session.request('GET', url, headers=headers) as response:
                try:
                    await response.json()
                    raise Exception(f'Athorization error in "{url}"')
                except:
                    auth_token = await response.text()
        except: 
            raise Exception(f'Athorization error in "{url}"')
        if auth_token:
            await self.async_set_default_headers({"authorization": f"Bearer {auth_token}"})
    
    def sync_request(self, method: str, endpoint: str, return_json: bool = True, **kwargs):
        if 'authorization' not in self._default_headers:
            self.sync_authorization()
        url = self._build_url(endpoint)
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self.timeout
        response = self.sync_session.request(method.upper(), url, **kwargs)
        return self._handle_response(response, return_json)
    
    async def async_request(self, method: str, endpoint: str, return_json: bool = True, **kwargs):
        if 'authorization' not in self._default_headers:
            await self.async_authorization()
        url = self._build_url(endpoint)
        async with self.async_session.request(method.upper(), url, **kwargs) as response:
            return await self._handle_async_response(response, return_json)
        
    def sync_set_webhook(self, channel_id: str, webhook_url: str):
        """
        Синхронная установка webhook URL для канала.
        
        Args:
            channel_id (str): ID канала
            webhook_url (str): URL для получения webhook уведомлений
            
        Returns:
            dict: Результат операции
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/set_webhook
        """
        json = {"webhookUrl": webhook_url}
        return self.sync_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/{channel_id}/webhook',
            json=json
        ) 
    
    async def async_set_webhook(self, channel_id: str, webhook_url: str):
        """
        Асинхронная установка webhook URL для канала.
        
        Args:
            channel_id (str): ID канала
            webhook_url (str): URL для получения webhook уведомлений
            
        Returns:
            dict: Результат операции
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/set_webhook
        """
        json = {"webhookUrl": webhook_url}
        return await self.async_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/{channel_id}/webhook',
            json=json
        ) 
    
    def sync_channel_post_file(self, channel_id: str, file_path: str):
        """
        Синхронная загрузка файла в канал.
        
        Args:
            channel_id (str): ID канала
            file_path (str): Путь к файлу для загрузки
            
        Returns:
            dict: Информация о загруженном файле
            
        Raises:
            FileNotFoundError: Если файл не найден
            Exception: Если загрузка не удалась
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/post_api_ext_v2_services__serviceId__channels__channelId__files
        """
        file_name = file_path.split(os.path.sep)[-1]
        mime_type, _ = mimetypes.guess_type(file_path)
        data = MultipartEncoder(fields={'file': (file_name, open(file_path, 'rb'), mime_type)})
        headers = self._default_headers.copy()
        headers['Content-Type'] = data.content_type
        self._default_headers.update(headers)
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/channels/{channel_id}/files',
            data=data,
        ) 
        headers['Content-Type'] = 'application/json'
        self._default_headers.update(headers)
        return resp
    
    async def async_channel_post_file(self, channel_id: str, file_path: str):
        """
        Асинхронная загрузка файла в канал.
        
        Args:
            channel_id (str): ID канала
            file_path (str): Путь к файлу для загрузки
            
        Returns:
            dict: Информация о загруженном файле
            
        Raises:
            FileNotFoundError: Если файл не найден
            Exception: Если загрузка не удалась
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/post_api_ext_v2_services__serviceId__channels__channelId__files
        """
        file_name = file_path.split(os.path.sep)[-1]
        mime_type, _ = mimetypes.guess_type(file_path)
        async with aiofiles.open(file_path, 'rb') as file:
            file_data = await file.read()
        data = FormData()
        data.add_field(
            'file',
            file_data,
            filename=file_name,
            content_type=mime_type
        )
        original_headers = self._default_headers.copy()
        headers = original_headers.copy()
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/channels/{channel_id}/files',
            data=data,
            headers=headers 
        )
        return resp
    
    def sync_channel_get_file(self, channel_id: str, file_id: str) -> bytes:
        """
        Синхронное получение файла из канала.
        
        Args:
            channel_id (str): ID канала
            file_id (str): ID файла
            
        Returns:
            dict: Данные файла
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/get_api_ext_v2_services__serviceId__channels__channelId__files__fileId_
        """
        resp = self.sync_request(
            'get',
            f'/api/ext/v2/services/{self.service_id}/channels/{channel_id}/files/{file_id}',
            return_json=False
        ) 
        return resp
    
    async def async_channel_get_file(self, channel_id: str, file_id: str) -> bytes:
        """
        Асинхронное получение файла из канала.
        
        Args:
            channel_id (str): ID канала
            file_id (str): ID файла
            
        Returns:
            dict: Данные файла
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/get_api_ext_v2_services__serviceId__channels__channelId__files__fileId_
        """
        resp = await self.async_request(
            'get',
            f'/api/ext/v2/services/{self.service_id}/channels/{channel_id}/files/{file_id}',
            return_json=False
        )
        return resp

    def sync_channel_question_async(self, channel_id: str, **kwargs):
        """
        Синхронная отправка вопроса в канал (асинхронная обработка).
        
        Args:
            channel_id (str): ID канала
            **kwargs: Параметры вопроса (см. QuestionModel)
            
        Returns:
            dict: Результат с ID вопроса или ошибками валидации
                - id (str): ID созданного вопроса
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/SendQuestionAsync
        """
        try: model = QuestionModel(**kwargs)
        except ValidationError as e:
            return {
                "id": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/{channel_id}/questionsAsync',
            json=json
        ) 
        return {"id": resp, "errors": None}
    
    async def async_channel_question_async(self, channel_id: str, **kwargs):
        """
        Асинхронная отправка вопроса в канал (асинхронная обработка).
        
        Args:
            channel_id (str): ID канала
            **kwargs: Параметры вопроса (см. QuestionModel)
            
        Returns:
            dict: Результат с ID вопроса или ошибками валидации
                - id (str): ID созданного вопроса
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/SendQuestionAsync
        """
        try: model = QuestionModel(**kwargs)
        except ValidationError as e:
            return {
                "id": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/{channel_id}/questionsAsync',
            json=json
        )
        return {"id": resp, "errors": None}
    
    def sync_close_conversation(self, conversation_id: str, **kwargs):
        """
        Синхронное закрытие диалога.
        
        Args:
            conversation_id (str): ID диалога
            **kwargs: Параметры закрытия (см. CloseConversationModel)
            
        Returns:
            dict: Результат операции или ошибки валидации
                - result: Результат закрытия
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/CloseConversation
        """
        try: model = CloseСonversationModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/conversations/{conversation_id}/close',
            json=json
        ) 
        return {"result": resp, "errors": None}
    
    async def async_close_conversation(self, conversation_id: str, **kwargs):
        """
        Асинхронное закрытие диалога.
        
        Args:
            conversation_id (str): ID диалога
            **kwargs: Параметры закрытия (см. CloseConversationModel)
            
        Returns:
            dict: Результат операции или ошибки валидации
                - result: Результат закрытия
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/CloseConversation
        """
        try: model = CloseСonversationModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/conversations/{conversation_id}/close',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_get_conversation_suggestions(self, conversation_id: str):
        """
        Синхронное получение предложений для диалога.
        
        Args:
            conversation_id (str): ID диалога
            
        Returns:
            dict: Список предложений и ошибки
                - result: Список предложений
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/GetSuggestions
        """
        resp = self.sync_request(
            'get',
            f'/api/ext/v2/services/{self.service_id}/conversations/{conversation_id}/suggestions',
        ) 
        return {"result": resp, "errors": None}
    
    async def async_get_conversation_suggestions(self, conversation_id: str):
        """
        Асинхронное получение предложений для диалога.
        
        Args:
            conversation_id (str): ID диалога
            
        Returns:
            dict: Список предложений и ошибки
                - result: Список предложений
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/GetSuggestions
        """
        resp = await self.async_request(
            'get',
            f'/api/ext/v2/services/{self.service_id}/conversations/{conversation_id}/suggestions',
        )
        return {"result": resp, "errors": None}
    
    def sync_get_conversations(self, **kwargs):
        """
        Синхронное получение списка диалогов с фильтрацией.
        
        Args:
            **kwargs: Параметры фильтрации (см. GetConversationsModel)
                - tsFrom (datetime): Начало периода
                - tsTo (datetime): Конец периода
                - limit (int): Лимит записей
                - page (int): Номер страницы
                - conversationStatusList (list): Список статусов
                - channelUserQuery (str): Поиск по пользователю
                - и другие...
            
        Returns:
            dict: Результат с данными диалогов или ошибками
                - result: Список диалогов
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/GetConversations
        """
        try: model = GetConversationsModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/conversations',
            json=json
        ) 
        return {"result": resp, "errors": None}
    
    async def async_get_conversations(self, **kwargs):
        """
        Асинхронное получение списка диалогов с фильтрацией.
        
        Args:
            **kwargs: Параметры фильтрации (см. GetConversationsModel)
                - tsFrom (datetime): Начало периода
                - tsTo (datetime): Конец периода
                - limit (int): Лимит записей
                - page (int): Номер страницы
                - conversationStatusList (list): Список статусов
                - channelUserQuery (str): Поиск по пользователю
                - и другие...
            
        Returns:
            dict: Результат с данными диалогов или ошибками
                - result: Список диалогов
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/GetConversations
        """
        try: model = GetConversationsModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/conversations',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_get_conversation(self, conversation_id):
        """
        Синхронное получение информации о конкретном диалоге.
        
        Args:
            conversation_id (str): ID диалога
            
        Returns:
            dict: Данные диалога и ошибки
                - result: Информация о диалоге
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/GetConversation
        """
        resp = self.sync_request(
            'get',
            f'/api/ext/v2/services/{self.service_id}/conversations/{conversation_id}',
        ) 
        return {"result": resp, "errors": None}
    
    async def async_get_conversation(self, conversation_id):
        """
        Асинхронное получение информации о конкретном диалоге.
        
        Args:
            conversation_id (str): ID диалога
            
        Returns:
            dict: Данные диалога и ошибки
                - result: Информация о диалоге
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переписка/GetConversation
        """
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/services/{self.service_id}/conversations/{conversation_id}',
        )
        return {"result": resp, "errors": None}
    
    def sync_create_delayed_delivery(self, **kwargs):
        """
        Синхронное создание отложенной рассылки сообщения.
        
        Args:
            **kwargs: Параметры рассылки (см. PostDelayedDeliveryModel)
                - serviceId (str): ID сервиса (автоподстановка)
                - message (str): Текст сообщения
                - scheduleTime (datetime): Время отправки
                - recipient (dict): Информация о получателе
                - и другие...
            
        Returns:
            dict: Результат создания рассылки или ошибки
                - result: Данные созданной рассылки
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/CreateDelivery
        """
        if 'serviceId' not in kwargs.keys(): kwargs['serviceId'] = self.service_id
        try: model = PostDelayedDeliveryModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/delayedDelivery',
            json=json
        ) 
        return {"result": resp, "errors": None}
    
    async def async_create_delayed_delivery(self, **kwargs):
        """
        Асинхронное создание отложенной рассылки сообщения.
        
        Args:
            **kwargs: Параметры рассылки (см. PostDelayedDeliveryModel)
                - serviceId (str): ID сервиса (автоподстановка)
                - message (str): Текст сообщения
                - scheduleTime (datetime): Время отправки
                - recipient (dict): Информация о получателе
                - и другие...
            
        Returns:
            dict: Результат создания рассылки или ошибки
                - result: Данные созданной рассылки
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/CreateDelivery
        """
        try: model = PostDelayedDeliveryModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/delayedDelivery',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_get_delayed_delivery_status(self, delayed_delivery_id):
        """
        Синхронная проверка статуса отложенной рассылки.
        
        Args:
            delayed_delivery_id (str): ID отложенной рассылки
            
        Returns:
            dict: Статус рассылки и ошибки
                - result: Статус рассылки
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/get_api_ext_v2_delayedDelivery_status
        """
        resp = self.sync_request(
            'get',
            f'/api/ext/v2/delayedDelivery',
            params={'delayedDeliveryId': delayed_delivery_id}
        ) 
        return {"result": resp, "errors": None}
    
    async def async_get_delayed_delivery_status(self, delayed_delivery_id):
        """
        Асинхронная проверка статуса отложенной рассылки.
        
        Args:
            delayed_delivery_id (str): ID отложенной рассылки
            
        Returns:
            dict: Статус рассылки и ошибки
                - result: Статус рассылки
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/get_api_ext_v2_delayedDelivery_status
        """
        resp = await self.async_request(
            'get',
            f'/api/ext/v2/delayedDelivery',
            params={'delayedDeliveryId': delayed_delivery_id}
        )
        return {"result": resp, "errors": None}
    
    def sync_get_delayed_delivery(self, delayed_delivery_id):
        """
        Синхронное получение информации об отложенной рассылки.
        
        Args:
            delayed_delivery_id (str): ID отложенной рассылки
            
        Returns:
            dict: Данные рассылки и ошибки
                - result: Информация о доставке
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/GetDelivery
        """
        resp = self.sync_request(
            'get',
            f'/api/ext/v2/delayedDelivery/{delayed_delivery_id}'
        ) 
        return {"result": resp, "errors": None}
    
    async def async_get_delayed_delivery(self, delayed_delivery_id):
        """
        Асинхронное получение информации об отложенной рассылки.
        
        Args:
            delayed_delivery_id (str): ID отложенной рассылки
            
        Returns:
            dict: Данные рассылки и ошибки
                - result: Информация о доставке
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/GetDelivery
        """
        resp = await self.async_request(
            'get',
            f'/api/ext/v2/delayedDelivery/{delayed_delivery_id}'
        )
        return {"result": resp, "errors": None}
    
    def sync_delete_delayed_delivery(self, delayed_delivery_id):
        """
        Синхронное удаление отложенной рассылки.
        
        Args:
            delayed_delivery_id (str): ID отложенной рассылки
            
        Returns:
            dict: Результат удаления и ошибки
                - result: Результат операции
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/DeleteDelivery
        """
        resp = self.sync_request(
            'delete',
            f'/api/ext/v2/delayedDelivery/{delayed_delivery_id}'
        ) 
        return {"result": resp, "errors": None}
    
    async def async_delete_delayed_delivery(self, delayed_delivery_id):
        """
        Асинхронное удаление отложенной рассылки.
        
        Args:
            delayed_delivery_id (str): ID отложенной рассылки
            
        Returns:
            dict: Результат удаления и ошибки
                - result: Результат операции
                - errors (list): Список ошибок
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Рассылки/DeleteDelivery
        """
        resp = await self.async_request(
            'delete',
            f'/api/ext/v2/delayedDelivery/{delayed_delivery_id}'
        )
        return {"result": resp, "errors": None}
    
    def sync_edit_vars(self, vars: list):
        """
        Синхронное редактирование переменных сервиса.
        
        Args:
            vars (list): Список переменных для обновления
                Каждый элемент должен содержать:
                - name (str): Имя переменной
                - value (str): Значение переменной
            
        Returns:
            dict: Результат обновления или ошибки
                - result: Результат операции
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переменные%20сервиса/EditServiceVars
        """
        try: model = [VarModel(**item) for item in vars]
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = [m.model_dump() for m in model]
        resp = self.sync_request(
            'put',
            f'/api/ext/v2/services/{self.service_id}/vars',
            json=json
        ) 
        return {"result": resp, "errors": None}
    
    async def async_edit_vars(self, vars: list):
        """
        Асинхронное редактирование переменных сервиса.
        
        Args:
            vars (list): Список переменных для обновления
                Каждый элемент должен содержать:
                - name (str): Имя переменной
                - value (str): Значение переменной
            
        Returns:
            dict: Результат обновления или ошибки
                - result: Результат операции
                - errors (list): Список ошибок валидации
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Переменные%20сервиса/EditServiceVars
        """
        try: model = [VarModel(**item) for item in vars]
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = [m.model_dump() for m in model]
        resp = await self.async_request(
            'put',
            f'/api/ext/v2/services/{self.service_id}/vars',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_closed_conversations_count_report(self, **kwargs):
        """
        Синхронное получение отчета по количеству закрытых диалогов.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета об уровне автоматизации
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_automation_level_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByWeek",
            ...     additionalGrouping="ByGroup"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_operators_status
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/number',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_closed_conversations_count_report(self, **kwargs):
        """
        Асинхронное получение отчета по количеству закрытых диалогов.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета об уровне автоматизации
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_automation_level_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByWeek",
            ...     additionalGrouping="ByGroup"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_operators_status
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/number',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_conversations_automation_level_report(self, **kwargs):
        """
        Синхронное получение отчета по уровню автоматизации диалогов.
        
        Отчет показывает статистику по автоматизации обработки диалогов,
        включая процент автоматически закрытых диалогов и распределение
        по уровням автоматизации.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета об уровне автоматизации
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_automation_level_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByWeek",
            ...     additionalGrouping="ByGroup"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_automation_level
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/automation-level',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_automation_level_report(self, **kwargs):
        """
        Асинхронное получение отчета по уровню автоматизации диалогов.
        
        Отчет показывает статистику по автоматизации обработки диалогов,
        включая процент автоматически закрытых диалогов и распределение
        по уровням автоматизации.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета об уровне автоматизации
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_automation_level_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByWeek",
            ...     additionalGrouping="ByGroup"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_automation_level
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/automation-level',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_conversations_kb_usage_report(self, **kwargs):
        """
        Синхронное получение отчета об абсолютные и относительные данные об использовании 
        баз знаний в автоматических ответах бот и рекомендациях операторам.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по CSAT/DSAT
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_csat_dsat_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_knowledge_base_usage
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/knowledge-base-usage',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_kb_usage_report(self, **kwargs):
        """
        Синхронное получение отчета об абсолютные и относительные данные об использовании 
        баз знаний в автоматических ответах бот и рекомендациях операторам.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по CSAT/DSAT
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_csat_dsat_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_knowledge_base_usage
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/knowledge-base-usage',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_conversations_rate_csat_dsat_report(self, **kwargs):
        """
        Синхронное получение отчета по CSAT (Customer Satisfaction) и DSAT (Customer Dissatisfaction).
        
        Отчет показывает уровень удовлетворенности клиентов на основе оценок,
        полученных после закрытия диалогов. CSAT - процент положительных оценок,
        DSAT - процент отрицательных оценок.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по CSAT/DSAT
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_csat_dsat_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_rate_csat_dsat
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/rate-csat-dsat',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_rate_csat_dsat_report(self, **kwargs):
        """
        Асинхронное получение отчета по CSAT (Customer Satisfaction) и DSAT (Customer Dissatisfaction).
        
        Отчет показывает уровень удовлетворенности клиентов на основе оценок,
        полученных после закрытия диалогов. CSAT - процент положительных оценок,
        DSAT - процент отрицательных оценок.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по CSAT/DSAT
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_csat_dsat_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_rate_csat_dsat
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/rate-csat-dsat',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_conversations_rate_distribution_report(self, **kwargs):
        """
        Синхронное получение отчета по распределению оценок диалогов.
        
        Отчет показывает детальное распределение всех полученных оценок
        (например, 1-5 звезд или другую шкалу оценок) с разбивкой по
        выбранным параметрам группировки.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по распределению оценок
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_distribution_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByMonth"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_rate_distribution
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/rate-distribution',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_rate_distribution_report(self, **kwargs):
        """
        Асинхронное получение отчета по распределению оценок диалогов.
        
        Отчет показывает детальное распределение всех полученных оценок
        (например, 1-5 звезд или другую шкалу оценок) с разбивкой по
        выбранным параметрам группировки.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по распределению оценок
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_distribution_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByMonth"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_rate_distribution
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/rate-distribution',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_conversations_rate_coverage_report(self, **kwargs):
        """
        Синхронное получение отчета по покрытию оценками.
        
        Отчет показывает процент диалогов, которые получили оценку
        от клиентов, от общего количества закрытых диалогов.
        Помогает анализировать вовлеченность клиентов в процесс оценки.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по покрытию оценками
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_coverage_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByChannel"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_rate_coverage
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/rate-coverage',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_rate_coverage_report(self, **kwargs):
        """
        Асинхронное получение отчета по покрытию оценками.
        
        Отчет показывает процент диалогов, которые получили оценку
        от клиентов, от общего количества закрытых диалогов.
        Помогает анализировать вовлеченность клиентов в процесс оценки.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по покрытию оценками
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_rate_coverage_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByChannel"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_rate_coverage
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/rate-coverage',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_conversations_afrt_timing_report(self, **kwargs):
        """
        Синхронное получение отчета по времени ответа AutoFAQ (AFRT - AutoFAQ Response Time).
        
        Отчет показывает статистику по времени, которое требуется системе AutoFAQ
        для генерации ответов на вопросы пользователей. Включает среднее время
        ответа, медиану, процентили и другие метрики.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа AutoFAQ
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_afrt_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByDay"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_afrt_timing
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/afrt-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_afrt_timing_report(self, **kwargs):
        """
        Асинхронное получение отчета по времени ответа AutoFAQ (AFRT - AutoFAQ Response Time).
        
        Отчет показывает статистику по времени, которое требуется системе AutoFAQ
        для генерации ответов на вопросы пользователей. Включает среднее время
        ответа, медиану, процентили и другие метрики.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа AutoFAQ
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_afrt_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByDay"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_afrt_timing
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/afrt-timing',
            json=json
        )
        return {"result": resp, "errors": None}

    def sync_conversations_art_timing_report(self, **kwargs):
        """
        Синхронное получение отчета по времени ответа операторов (ART - Agent Response Time).
        
        Отчет показывает статистику по времени, которое требуется операторам
        для ответа на сообщения пользователей. Включает среднее время ответа,
        медиану, процентили и распределение по операторам или группам.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_art_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_art_timing
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/conversations/art-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_conversations_art_timing_report(self, **kwargs):
        """
        Асинхронное получение отчета по времени ответа операторов (ART - Agent Response Time).
        
        Отчет показывает статистику по времени, которое требуется операторам
        для ответа на сообщения пользователей. Включает среднее время ответа,
        медиану, процентили и распределение по операторам или группам.
        
        Args:
            **kwargs: Параметры отчета (см. ConversationsCountReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "ByOperator" - по операторам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
                - documentTags (list, optional): Список тегов документов для фильтрации
                - groups (list, optional): Список ID групп для фильтрации
                - filters (list, optional): Дополнительные фильтры
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_conversations_art_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20диалогам/post_api_ext_v2_reports_conversations_art_timing
        """
        try: model = ConversationsCountReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/conversations/art-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_status_report(self, **kwargs):
        """
        Синхронное получение отчета по статусам операторов.
        
        Отчет показывает распределение операторов по статусам (онлайн, офлайн, перерыв и т.д.)
        за выбранный период времени с возможностью группировки по различным параметрам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по статусам операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_status_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByDay",
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_status
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/status',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_status_report(self, **kwargs):
        """
        Асинхронное получение отчета по статусам операторов.
        
        Отчет показывает распределение операторов по статусам (онлайн, офлайн, перерыв и т.д.)
        за выбранный период времени с возможностью группировки по различным параметрам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по статусам операторов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_status
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/status',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_productivity_absolute_report(self, **kwargs):
        """
        Синхронное получение отчета по абсолютной продуктивности операторов.
        
        Отчет показывает абсолютные показатели продуктивности операторов:
        количество обработанных диалогов, закрытых диалогов, отправленных сообщений
        и другие количественные метрики за выбранный период.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по абсолютной продуктивности
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_productivity_absolute_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_productivity_absolute
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/productivity/absolute',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_productivity_absolute_report(self, **kwargs):
        """
        Асинхронное получение отчета по абсолютной продуктивности операторов.
        
        Отчет показывает абсолютные показатели продуктивности операторов:
        количество обработанных диалогов, закрытых диалогов, отправленных сообщений
        и другие количественные метрики за выбранный период.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по абсолютной продуктивности
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_productivity_absolute
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/productivity/absolute',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_productivity_relative_report(self, **kwargs):
        """
        Синхронное получение отчета по относительной продуктивности операторов.
        
        Отчет показывает относительные показатели продуктивности операторов:
        среднее время обработки диалога, процент автоматизации, эффективность
        и другие относительные метрики в сравнении между операторами или группами.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по относительной продуктивности
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_productivity_relative_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByWeek"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_productivity_relative
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/productivity/relative',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_productivity_relative_report(self, **kwargs):
        """
        Асинхронное получение отчета по относительной продуктивности операторов.
        
        Отчет показывает относительные показатели продуктивности операторов:
        среднее время обработки диалога, процент автоматизации, эффективность
        и другие относительные метрики в сравнении между операторами или группами.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по относительной продуктивности
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_productivity_relative
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/productivity/relative',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_occupancy_report(self, **kwargs):
        """
        Синхронное получение отчета по загрузке (occupancy) операторов.
        
        Отчет показывает уровень загрузки операторов - процент времени,
        который операторы фактически работали с диалогами от общего
        рабочего времени за выбранный период.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по загрузке операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_occupancy_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByGroup"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_occupancy
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/occupancy',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_occupancy_report(self, **kwargs):
        """
        Асинхронное получение отчета по загрузке (occupancy) операторов.
        
        Отчет показывает уровень загрузки операторов - процент времени,
        который операторы фактически работали с диалогами от общего
        рабочего времени за выбранный период.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по загрузке операторов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_occupancy
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/occupancy',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_concurrency_report(self, **kwargs):
        """
        Синхронное получение отчета по параллельности работы операторов.
        
        Отчет показывает среднее количество одновременных диалогов, которые 
        операторы ведут параллельно, а также максимальные значения параллельности 
        для анализа нагрузки и эффективности работы операторов.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по параллельности работы операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_concurrency_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByDay"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_concurrency
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/concurrency',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_concurrency_report(self, **kwargs):
        """
        Асинхронное получение отчета по параллельности работы операторов.
        
        Отчет показывает среднее количество одновременных диалогов, которые 
        операторы ведут параллельно, а также максимальные значения параллельности 
        для анализа нагрузки и эффективности работы операторов.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по параллельности работы операторов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_concurrency
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/concurrency',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_afrt_timing_report(self, **kwargs):
        """
        Синхронное получение отчета по времени ответа AutoFAQ для операторов (AFRT).
        
        Отчет показывает статистику по времени, которое требуется системе AutoFAQ
        для генерации ответов на вопросы пользователей в диалогах, которые 
        обрабатываются конкретными операторами или группами операторов.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа AutoFAQ для операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_afrt_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByOperator"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_afrt_operator_timing
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/afrt-operator-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_afrt_timing_report(self, **kwargs):
        """
        Асинхронное получение отчета по времени ответа AutoFAQ для операторов (AFRT).
        
        Отчет показывает статистику по времени, которое требуется системе AutoFAQ
        для генерации ответов на вопросы пользователей в диалогах, которые 
        обрабатываются конкретными операторами или группами операторов.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа AutoFAQ для операторов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_afrt_operator_timing
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/afrt-operator-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_art_timing_report(self, **kwargs):
        """
        Синхронное получение отчета по времени ответа операторов (ART).
        
        Отчет показывает статистику по времени, которое требуется операторам
        для ответа на сообщения пользователей. Включает среднее время ответа,
        медиану, процентили и распределение по операторам или группам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_art_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByGroup"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_art_operator_timing
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/art-operator-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_art_timing_report(self, **kwargs):
        """
        Асинхронное получение отчета по времени ответа операторов (ART).
        
        Отчет показывает статистику по времени, которое требуется операторам
        для ответа на сообщения пользователей. Включает среднее время ответа,
        медиану, процентили и распределение по операторам или группам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по времени ответа операторов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_art_operator_timing
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/art-operator-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_aht_timing_report(self, **kwargs):
        """
        Синхронное получение отчета по среднему времени обработки диалогов (AHT).
        
        Отчет показывает Average Handling Time - среднее время, которое операторы
        затрачивают на полную обработку одного диалога от первого сообщения 
        до закрытия, включая все этапы работы с клиентом.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по среднему времени обработки диалогов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_aht_timing_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByWeek"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_aht_operator_timing
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/aht-operator-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_aht_timing_report(self, **kwargs):
        """
        Асинхронное получение отчета по среднему времени обработки диалогов (AHT).
        
        Отчет показывает Average Handling Time - среднее время, которое операторы
        затрачивают на полную обработку одного диалога от первого сообщения 
        до закрытия, включая все этапы работы с клиентом.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по среднему времени обработки диалогов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_aht_operator_timing
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/aht-operator-timing',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_afrt_sla_report(self, **kwargs):
        """
        Синхронное получение отчета по соблюдению SLA для времени ответа AutoFAQ.
        
        Отчет показывает процент диалогов, в которых время ответа системы AutoFAQ
        укладывается в установленные Service Level Agreement (SLA) нормы.
        Помогает оценить соответствие скорости работы бота требуемым стандартам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по соблюдению SLA для времени ответа AutoFAQ
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_afrt_sla_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     additionalGrouping="ByChannel"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_afrt_sla
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/afrt-sla',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_afrt_sla_report(self, **kwargs):
        """
        Асинхронное получение отчета по соблюдению SLA для времени ответа AutoFAQ.
        
        Отчет показывает процент диалогов, в которых время ответа системы AutoFAQ
        укладывается в установленные Service Level Agreement (SLA) нормы.
        Помогает оценить соответствие скорости работы бота требуемым стандартам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по соблюдению SLA для времени ответа AutoFAQ
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_afrt_sla
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/afrt-sla',
            json=json
        )
        return {"result": resp, "errors": None}
    
    def sync_operators_art_sla_report(self, **kwargs):
        """
        Синхронное получение отчета по соблюдению SLA для времени ответа операторов.
        
        Отчет показывает процент диалогов, в которых время ответа операторов
        укладывается в установленные Service Level Agreement (SLA) нормы.
        Помогает оценить соответствие скорости работы операторов требуемым стандартам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по соблюдению SLA для времени ответа операторов
                - errors (list): Список ошибок валидации или None
            
        Example:
            >>> report = client.sync_operators_art_sla_report(
            ...     dateRange={
            ...         "from": "2024-01-01T00:00:00Z",
            ...         "to": "2024-01-31T23:59:59Z"
            ...     },
            ...     dateGrouping="ByMonth"
            ... )
            
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_art_sla
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = self.sync_request(
            'post',
            f'/api/ext/v2/reports/operators/art-sla',
            json=json
        )
        return {"result": resp, "errors": None}
    
    async def async_operators_art_sla_report(self, **kwargs):
        """
        Асинхронное получение отчета по соблюдению SLA для времени ответа операторов.
        
        Отчет показывает процент диалогов, в которых время ответа операторов
        укладывается в установленные Service Level Agreement (SLA) нормы.
        Помогает оценить соответствие скорости работы операторов требуемым стандартам.
        
        Args:
            **kwargs: Параметры отчета (см. OperatorsReportModel)
                - dateRange (dict): Временной диапазон отчета
                    - from (datetime): Начало периода
                    - to (datetime): Конец периода
                - operators (list, optional): Список UUID операторов для фильтрации
                - groups (list, optional): Список UUID групп для фильтрации
                - dateGrouping (str): Группировка по дате. Варианты:
                    - "ByDay" - по дням
                    - "ByWeek" - по неделям  
                    - "ByMonth" - по месяцам
                    - "ByYear" - по годам
                - additionalGrouping (str): Дополнительная группировка. Варианты:
                    - "ByOperator" - по операторам
                    - "ByGroup" - по группам
                    - "ByChannel" - по каналам
                    - "None" - без группировки
                - knowledgeBases (list, optional): Список ID баз знаний для фильтрации
            
        Returns:
            dict: Результат с данными отчета или ошибками валидации
                - result: Данные отчета по соблюдению SLA для времени ответа операторов
                - errors (list): Список ошибок валидации или None
                
        Link: 
            https://app.swaggerhub.com/apis-docs/AutoFAQ.ai/external-api/2.1.4#/Отчеты%20по%20операторам/post_api_ext_v2_reports_operators_art_sla
        """
        try: model = OperatorsReportModel(**kwargs)
        except ValidationError as e:
            return {
                "results": None,
                "errors": e.errors()
            }
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            json = model.model_dump()
            if 'dateRange' in json:
                if json['dateRange']:
                    if 'from_' in json['dateRange']:
                        json['dateRange']['from'] = json['dateRange']['from_']
                        del json['dateRange']['from_']
        resp = await self.async_request(
            'post',
            f'/api/ext/v2/reports/operators/art-sla',
            json=json
        )
        return {"result": resp, "errors": None}


from pydantic import BaseModel, Field, field_validator, ConfigDict, model_validator
from typing import List, Optional, Literal
from datetime import datetime
import uuid

REASONS_LITERAL = Literal[
    'ClosedByBot', 
    'ClosedByOperator', 
    'ClosedByOperatorWithBot', 
    'ClosedByTimer', 
    'Active', 
    'OnOperator',  
    'AssignedToOperator', 
]
STATUSES = [
    'ClosedByBot', 
    'ClosedByOperator', 
    'ClosedByOperatorWithBot', 
    'ClosedByTimer', 
    'Active', 
    'OnOperator', 
    'AssignedToOperator', 
]

class _ChannelUser(BaseModel):
    id: str = Field(..., description="ID пользователя")
    login: str = Field(..., description="Логин пользователя")
    email: str = Field(..., description="Email пользователя")
    phone: str = Field(..., description="Телефон пользователя")
    fullName: str = Field(..., description="Полное имя пользователя")
    payload: dict = Field(default={}, description="Дополнительные данные пользователя")

    class Config:
        populate_by_name = True
     
class _Operator(BaseModel):
    email: str = Field(None, description="Email оператора")
    login: str = Field(None, description="Логин оператора")
    phone: str = Field(None, description="Телефон оператора")
    fullName: str = Field(None, description="Полное имя оператора")
    payload: dict = Field(default={}, description="Дополнительные данные оператора")
    

class QuestionModel(BaseModel):
    dt: datetime = Field(..., description="Дата и время сообщения")
    text: str = Field(..., description="Текст сообщения")
    fileIds: List[str] = Field(default=[], description="Список ID файлов")
    channelUser: _ChannelUser = Field(..., description="Информация о пользователе канала")
    
    @field_validator('fileIds')
    def validate_file_ids(cls, v):
        try:
            uuid.UUID(v)
            return v
        except ValueError:
            raise ValueError(f"Invalid UUID format: {v}")

    @field_validator('dt')
    def validate_dt_format(cls, v):
        if isinstance(v, str):
            try:
                return datetime.fromisoformat(v.replace('Z', '+00:00')).isoformat().split('+')[0]+'Z'
            except ValueError:
                raise ValueError("Invalid datetime format. Expected ISO 8601 format")
        elif isinstance(v, datetime):
            return v.isoformat().split('+')[0]+'Z'
        return v

    # class Config:
    #     json_encoders = {
    #         datetime: lambda v: v.isoformat().replace('+00:00', 'Z')
    #     }
    #     populate_by_name = True
    
    
class CloseСonversationModel(BaseModel):
    reason: REASONS_LITERAL = Field(default='ClosedByBot', description="Причина закрытия")
    closeToAutofaqServiceId: int = Field(None, description="ID сервиса AutoFAQ")
    closeToAutofaqDocumentId: int = Field(None, description="ID документа AutoFAQ")
    operator: _Operator = Field(None, description="Информация об операторе")
    
    
class GetConversationsModel(BaseModel):
    tsFrom: Optional[datetime] = Field(None, description="Начальная дата периода")
    tsTo: Optional[datetime] = Field(None, description="Конечная дата периода")
    limit: Optional[int] = Field(50, ge=1, le=100, description="Лимит записей (1-100)")
    page: Optional[int] = Field(1, ge=1, description="Номер страницы")
    orderDirection: Optional[Literal["Asc", "Desc"]] = Field("Desc", description="Направление сортировки")
    conversationStatusList: Optional[List[str]] = Field(None, description="Список статусов диалогов")
    channelUserQuery: Optional[str] = Field(None, description="Поиск по пользователю канала")
    participatingOperatorList: Optional[List[str]] = Field(None, description="Список операторов")
    themeList: Optional[List[str]] = Field(None, description="Список тем")
    groupList: Optional[List[str]] = Field(None, description="Список групп")

    @field_validator('tsTo')
    def validate_date_range(cls, v, values):
        if 'tsFrom' in values and values['tsFrom'] and v:
            if v <= values['tsFrom']:
                raise ValueError('tsTo must be greater than tsFrom')
        return v

    @field_validator('conversationStatusList')
    def validate_conversation_status(cls, v):
        for i in v:
            if i not in STATUSES:
                raise ValueError(f'Status must be one of: {", ".join(STATUSES)}')
        return v
    
    
class _Interval(BaseModel):
    start: datetime = Field(..., description="Начало интервала")
    end: datetime = Field(..., description="Конец интервала")

    model_config = ConfigDict(from_attributes=True)

    @model_validator(mode='after')
    def validate_end_after_start(self):
        if self.end <= self.start:
            raise ValueError('end must be after start')
        return self


class _EveryWeekPlan(BaseModel):
    daysOfWeek: List[Literal["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]] = Field(
        ..., description="Дни недели"
    )
    time: str = Field(..., pattern=r'^([01]?[0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]$', description="Время в формате HH:MM:SS")
    timezone: str = Field(..., description="Часовой пояс")

    model_config = ConfigDict(from_attributes=True)
    
    
class _Plan(BaseModel):
    everyWeek: _EveryWeekPlan = Field(..., description="План на каждую неделю")

    model_config = ConfigDict(from_attributes=True)


class _TextContent(BaseModel):
    value: str = Field(..., description="Текст сообщения")
    type: Literal["text"] = Field("text", description="Тип контента")

    model_config = ConfigDict(from_attributes=True)


class _FilterItem(BaseModel):
    userId: Optional[str] = Field(None, description="ID пользователя")
    userFullName: Optional[str] = Field(None, description="Полное имя пользователя")
    userPhone: Optional[str] = Field(None, description="Телефон пользователя")
    userPayload_studentId: Optional[str] = Field(None, alias="userPayload.studentId", description="Student ID из payload")

    model_config = ConfigDict(
        from_attributes=True,
        populate_by_name=True
    )


class PostDelayedDeliveryModel(BaseModel):
    serviceId: str = Field(..., description="ID сервиса")
    groupId: str = Field(None, description="ID группы")
    channelId: str = Field(..., description="ID канала")
    state: Literal["Active", "Inactive", "Draft"] = Field(..., description="Статус рассылки")
    name: str = Field(None, min_length=1, max_length=255, description="Название рассылки")
    title: str = Field(..., min_length=1, max_length=255, description="Заголовок рассылки")
    description: Optional[str] = Field(None, description="Описание рассылки")
    interval: _Interval = Field(None, description="Интервал рассылки")
    plan: _Plan = Field(None, description="План рассылки")
    text: _TextContent = Field(..., description="Текст сообщения")
    filterType: Literal["static", "dynamic",] = Field(..., description="Тип фильтра")
    filter: List[_FilterItem] = Field(None, description="Фильтр пользователей")

    model_config = ConfigDict(
        from_attributes=True,
        populate_by_name=True,
        str_strip_whitespace=True
    )


class VarModel(BaseModel):
    name: str = Field(..., min_length=1, max_length=100, description="Название")
    value: str = Field(..., min_length=1, max_length=500, description="Значение")
    
    model_config = ConfigDict(
        str_strip_whitespace=True,
        from_attributes=True
    )

class _DateRange(BaseModel):
    from_: datetime = Field(..., alias="from")
    to: datetime

    class Config:
        populate_by_name = True
        
    @field_validator('from_')
    def validate_dt_format(cls, v, values):
        if isinstance(v, str):
            try:
                return datetime.fromisoformat(v.replace('Z', '+00:00')).isoformat().split('+')[0]+'Z'
            except ValueError:
                raise ValueError("Invalid datetime format. Expected ISO 8601 format")
        elif isinstance(v, datetime):
            return v.isoformat().split('+')[0]+'Z'
        return v

    @field_validator('to')
    def validate_to_date(cls, v, values):
        if isinstance(v, str):
            try:
                return datetime.fromisoformat(v.replace('Z', '+00:00')).isoformat().split('+')[0]+'Z'
            except ValueError:
                raise ValueError("Invalid datetime format. Expected ISO 8601 format")
        elif isinstance(v, datetime):
            return v.isoformat().split('+')[0]+'Z'
        return v
    
    
class ConversationsCountReportModel(BaseModel):
    dateRange: _DateRange = Field(..., description="Интервал времени")
    dateGrouping: Literal["ByDay", "ByWeek", "ByMonth", "ByYear"] = Field("ByWeek")
    additionalGrouping: Literal["ByGroup", "ByChannel", "ByOperator"] = Field("ByGroup")
    knowledgeBases: list = Field(default=[])
    documentTags: list = Field(default=[])
    groups: list = Field(default=[])
    filters: list = Field(default=[])


class OperatorsReportModel(BaseModel):
    dateRange: _DateRange = Field(..., description="Временной диапазон отчета")
    operators: list = Field(default=[], description="Список операторов для фильтрации")
    groups: list = Field(default=[], description="Список групп для фильтрации")
    dateGrouping: Literal["ByDay", "ByWeek", "ByMonth", "ByYear"] = Field(
        "ByDay", 
        description="Группировка по дате"
    )
    additionalGrouping: Literal["ByOperator", "ByGroup", "ByChannel", "None"] = Field(
        "ByGroup", 
        description="Дополнительная группировка"
    )
    knowledgeBases: list = Field(default=[], description="Список баз знаний для фильтрации")